<?php
// This bot can be used as a transfer target and will ask the user all
// the configured $questions. Supported types are 'text' and 'range'
// When all the questions are answered, the chat will be transferred
// back for human handling. This can be used to ask the user a set of
// data questions without burdening or taking agent time.

$DEBUG=0;

$config = array(
   'last_action'        => 'transfer',
   'invalid_answer_msg' => 'No entendí la respuesta.',
   'close_msg'          => 'Gracias!',
   'transfer_msg'       => 'Gracias, lo contactaré con un asistente humano'
);

$questions = array(
    array("body"=> "Cuál es su nombre?", "type"=>"text"),
    array("body"=> "Del 1 al 5, cómo nos calificaría, siendo 1 muy malo y 5 muy bueno?", "type"=>"range", "extra"=>"1,2,3,4,5"),
    array("body"=> "Qué día está libre?", "type"=>"text"),
    array("body"=> "Por favor elija una opción:\n1) Consulta General\n2) Retiro de puntos\n3) Vacuna o Inyectable", "type"=>"menu", "responses"=>array("1"=>"1","2"=>"2","3"=>"3")),
    array("body"=> "Muy bien, ahora elija la especialidad:\nA) Consulta general\nB) Consulta Ortopédica\nC) Consulta de control", "type"=>"menu", "responses"=>array("A"=>"99","B"=>"99","C"=>"99")),
    array("body"=> "Muy bien, que tipo?:\nA) Retiro de puntos de Cirugía Ortopédica\nB) Retiro de Puntos de otra cirugía\n", "type"=>"menu", "responses"=>array("A"=>"99","B"=>"99")),
    array("body"=> "Muy bien, que tipo de vacuna?:\nA) Vacunas anuales\nB) Vacunas de cachorros\nC) Inyectable de control", "type"=>"menu", "responses"=>array("A"=>"99","B"=>"99","C"=>"99")),
);

$json = file_get_contents('php://input');
$data = json_decode($json,1);

if(!isset($data)) { exit; }

$validate_command = '';

if($data['state']=='') {
    $step=0;
} else {
    // Not first interaction, validate answer from previous step
    $step=intval($data['state']);
    $answer=$data['body'];
    $prev_step = $step-1;
    $validate_command = validate_answer($prev_step,$answer);
    log_debug("validate question $prev_step, ($answer) return comand $validate_command");
}

if($validate_command=='[wrong]') {
    $msg    = retrieve_question($step-1);
    $return = array("body"=>$config['invalid_answer_msg']." ".$msg,"attach"=>'',"state"=>$step);
    echo json_encode($return);
    die();
} else if($validate_command=='[close]') {
    $return = array("body"=>$config['close_msg']." [close]","attach"=>'',"state"=>'');
    echo json_encode($return);
    die();
} else if($validate_command=='[transfer]') {
    $return = array("body"=>$config['transfer_msg']." [transfer]","attach"=>'',"state"=>'');
    echo json_encode($return);
    die();
} else if(preg_match("/\[next=(\d+)\]/", $validate_command, $matches)) {
    $step = $matches[1];
    log_debug("next step from menu ".$step);
}

$msg = retrieve_question($step);
$next_step = $step+1;

log_debug("retrieve question $step returned $msg, set state to $next_step");

if($msg=='[close]')    $msg = $config['close_msg']." [close]";
if($msg=='[transfer]') $msg = $config['transfer_msg']." [transfer]";

$return = array("body"=>$msg,"attach"=>'',"state"=>$next_step);
echo json_encode($return);
exit;

function retrieve_question($n) {
    global $questions, $config;
    if(!isset($questions[$n])) return "[".$config['last_action']."]";
    return $questions[$n]['body'];
}

function validate_answer($n,$answer) {
    global $questions, $config;
    if(!isset($questions[$n])) {
        return "[".$config['last_action']."]";
    }
    if($questions[$n]['type']=='range') {
        $valid_answers = preg_split("/,/",$questions[$n]['extra']);
        if(!in_array($answer,$valid_answers)) {
            return "[wrong]";
        }
    } else
    if($questions[$n]['type']=='menu') {
        $valid_answers=array();
        foreach($questions[$n]['responses'] as $key=>$val) {
            $valid_answers[strtolower($key)]=$val;
        }
        if(!array_key_exists(strtolower($answer),$valid_answers)) {
            return "[wrong]";
        }
        $next_step = $valid_answers[strtolower($answer)];
        return "[next=$next_step]";
    }
    return '';
}

function log_debug($text) {
    global $DEBUG;
    $logfile = basename(__FILE__,'.php');
    if($DEBUG==1) {
        file_put_contents("/tmp/${logfile}.log",$text."\n",FILE_APPEND);
    }
}
